--circle collision test

local _lg = love.graphics

function love.load()
	speed = 100
	cx = 100
	cy = 100

	toggle = true

	width = 200
	height = 100

	lx1 = 20
	ly1 = 20
	lx2 = 400
	ly2 = 200
end

function love.update(dt)
	if love.keyboard.isDown("left") or love.keyboard.isDown("a") then
		cx = cx - speed * dt
	end
	if love.keyboard.isDown("right") or love.keyboard.isDown("d") then
		cx = cx + speed * dt
	end
	if love.keyboard.isDown("up") or love.keyboard.isDown("w")then
		cy = cy - speed * dt
	end
	if love.keyboard.isDown("down") or love.keyboard.isDown("s") then
		cy = cy + speed * dt
	end

	line_intersections = Line_Rect_Intersect(lx1,ly1,lx2,ly2,cx,cy,width, height)
end

function love.draw()
	_lg.setColor(255,0,0,200)
	_lg.rectangle('fill',cx,cy,width,height)
	_lg.setColor(0,200,0,200)
	_lg.line(lx1,ly1,lx2,ly2)
	_lg.print( #line_intersections .. " collisions.", 400,0)
	_lg.print(cx .. " " .. cy, 0, 0)

	for i,v in ipairs(line_intersections) do
		_lg.setColor(222,222,222,200)
		_lg.print(tostring(v.x) .. " " .. v.y, 400, 15*i)
		if v.x then
			_lg.circle('fill', v.x,v.y, 5)
		end
	end

	_lg.print("Press [space] to toggle tolerance.", 400, 200)
	_lg.print("Tolerance: " .. tostring(toggle), 400, 215)
end

function Line_Rect_Intersect(lx1, ly1, lx2, ly2, rx, ry, rw, rh)
	local intersections = {}
	--top line
	local x,y = findIntersect(lx1, ly1, lx2, ly2, rx, ry, rx + rw, ry, true, true, toggle)
	if true then table.insert(intersections, {x=x,y=y}) end
	--bottom line
	x,y = findIntersect(lx1, ly1, lx2, ly2, rx, ry + rh, rx + rw, ry + rh, true, true, toggle)
	if true then table.insert(intersections, {x=x,y=y}) end
	--left line
	x,y = findIntersect(lx1, ly1, lx2, ly2, rx, ry, rx, ry + rh, true, true, toggle)
	if true then table.insert(intersections, {x=x,y=y}) end
	--right line
	x,y = findIntersect(lx1, ly1, lx2, ly2, rx + rw, ry, rx + rw, ry + rh, true, true, toggle)
	if true then table.insert(intersections, {x=x,y=y}) end

	return intersections
end

-- Checks if two lines intersect (or line segments if seg is true)
-- Lines are given as four numbers (two coordinates)
function findIntersect(l1p1x,l1p1y, l1p2x,l1p2y, l2p1x,l2p1y, l2p2x,l2p2y, seg1, seg2, tol)
	-- added tolerance
	local tolerance = tol and 0.1 or 0
    local a1,b1,a2,b2 = l1p2y-l1p1y, l1p1x-l1p2x, l2p2y-l2p1y, l2p1x-l2p2x
    local c1,c2 = a1*l1p1x+b1*l1p1y, a2*l2p1x+b2*l2p1y
    local det = a1*b2 - a2*b1
    if det==0 then return false, "The lines are parallel." end
    local x,y = (b2*c1-b1*c2)/det, (a1*c2-a2*c1)/det
    if seg1 or seg2 then
        local min,max = math.min, math.max
        if seg1 and not (min(l1p1x,l1p2x) <= x + tolerance and x <= max(l1p1x,l1p2x) + tolerance and min(l1p1y,l1p2y) <= y + tolerance 
        	and y <= (max(l1p1y,l1p2y)) + tolerance) or
           seg2 and not (min(l2p1x,l2p2x) <= x + tolerance and x <= max(l2p1x,l2p2x) + tolerance and min(l2p1y,l2p2y) <= y + tolerance 
           	and y <= (max(l2p1y,l2p2y)) + tolerance) then
            return false, "The lines don't intersect."
        end
    end
    return x,y
end

function love.keypressed(key,code)
	if key == ' ' then
		toggle = not toggle
	end
end